@@ -120,6 +120,27 @@ class Tools:
         except:
             return dict(np.load(f).item())
 
+    def save_distributed_perpix_dic(self, dic, outdir, n=10000):
+        '''
+        :param dic:
+        :param outdir:
+        :param n: save to each file every n sample
+        :return:
+        '''
+        flag = 0
+        temp_dic = {}
+        for key in tqdm(dic, 'saving...'):
+            flag += 1
+            arr = dic[key]
+            arr = np.array(arr)
+            temp_dic[key] = arr
+            if flag % n == 0:
+                np.save(outdir + '/per_pix_dic_%03d' % (flag / n), temp_dic)
+                temp_dic = {}
+        np.save(outdir + '/per_pix_dic_%03d' % 0, temp_dic)
+
+        pass
+
     def load_df(self, f):
         df = pd.read_pickle(f)
         df = pd.DataFrame(df)
@@ -287,6 +308,32 @@ class Tools:
 
         pass
 
+    def interp_nan_climatology(self,vals):
+        vals = np.array(vals)
+        vals_reshape = np.reshape(vals,(-1,12))
+        vals_reshape_T = vals_reshape.T
+        month_mean = []
+        for m in vals_reshape_T:
+            mean = np.nanmean(m)
+            month_mean.append(mean)
+        nan_index = np.isnan(vals)
+        val_new = []
+        for i in range(len(nan_index)):
+            isnan = nan_index[i]
+            month = i % 12
+            interp_val = month_mean[month]
+            if isnan:
+                val_new.append(interp_val)
+            else:
+                val_new.append(vals[i])
+        val_new = np.array(val_new)
+        return val_new
+
+
+    def detrend_vals(self,vals):
+        return signal.detrend(vals)
+        pass
+
     def detrend_dic(self, dic):
         dic_new = {}
         for key in dic:
@@ -600,14 +647,26 @@ class Tools:
         data = []
         columns = []
         index = []
+        all_cols = []
         for key in dic:
             vals = dic[key]
+            for col in vals:
+                all_cols.append(col)
+        all_cols = list(set(all_cols))
+        all_cols.sort()
+        for key in dic:
+            vals = dic[key]
+            if len(vals) == 0:
+                continue
             vals_list = []
             col_list = []
             vals_list.append(key)
             col_list.append(key_col_str)
-            for col in vals:
-                val = vals[col]
+            for col in all_cols:
+                if not col in vals:
+                    val = np.nan
+                else:
+                    val = vals[col]
                 vals_list.append(val)
                 col_list.append(col)
             data.append(vals_list)
@@ -669,7 +728,7 @@ class Tools:
         df = df.dropna(how='all',subset=var_name_list)
         return df
 
-    def add_dic_to_df(self,df,dic,key_name):
+    def add_spatial_dic_to_df(self,df,dic,key_name):
         val_list = []
         for i,row in df.iterrows():
             pix = row['pix']
@@ -679,6 +738,7 @@ class Tools:
                 val = dic[pix]
             val_list.append(val)
         df[key_name] = val_list
+        return df
 
     def df_to_dic(self,df,key_str='__key__'):
         '''
@@ -690,6 +750,8 @@ class Tools:
         dic = {}
         for i, row in df.iterrows():
             key = row[key_str]
+            if key in dic:
+                raise UserWarning(f'"{key_str}" is not unique\ni.e. "{key}" is not unique')
             dic_i = {}
             for col in columns:
                 val = row[col]
@@ -739,6 +801,48 @@ class Tools:
         vals[vals<down] = np.nan
         return vals
 
+
+    def monthly_vals_to_annual_val(self,monthly_vals,grow_season=None,method='mean'):
+        '''
+        from
+        [1,2,3....,46,47,48]
+        to
+        ([1,2,...11,12]
+        [13,14,...23,24]
+        .
+        .
+        [37,38,...47,48])
+        to
+        [1,2,3,4]
+        :param monthly_vals:
+        :param grow_season: default is 1-12
+        :return:
+        '''
+        if grow_season == None:
+            grow_season = list(range(12))
+        else:
+            grow_season = np.array(grow_season,dtype=int)
+            grow_season = grow_season - 1
+            if grow_season[0] < 0:
+                raise UserWarning(f'Error grow_season:{grow_season}')
+        monthly_vals = np.array(monthly_vals)
+        monthly_vals_reshape = np.reshape(monthly_vals,(-1,12))
+        annual_val_list = []
+        for one_year_vals in monthly_vals_reshape:
+            one_year_vals_pick = Tools().pick_vals_from_1darray(one_year_vals,grow_season)
+            if method == 'mean':
+                annual_val = np.nanmean(one_year_vals_pick)
+            elif method == 'max':
+                annual_val = np.nanmax(one_year_vals_pick)
+            elif method == 'min':
+                annual_val = np.nanmin(one_year_vals_pick)
+            else:
+                raise UserWarning(f'method:{method} error')
+            annual_val_list.append(annual_val)
+        annual_val_list = np.array(annual_val_list)
+
+        return annual_val_list
+
 class SMOOTH:
     '''
     一些平滑算法
@@ -1895,7 +1999,7 @@ class Pre_Process:
                 if f.endswith('.tif'):
                     if f.split('.')[0] == d:
                         # print(d)
-                        array, originX, originY, pixelWidth, pixelHeight = ToRaster().raster2array(Tools().path_join(fdir, f))
+                        array, originX, originY, pixelWidth, pixelHeight = ToRaster().raster2array(join(fdir, f))
                         array = np.array(array, dtype=np.float)
                         # print np.min(array)
                         # print type(array)
@@ -2103,7 +2207,8 @@ class Pre_Process:
         MULTIPROCESS(self.kernel_cal_anomaly, params).run(process=4, process_or_thread='p',
                                                           desc='calculating anomaly...')
 
-    def clean_per_pix(self, fdir, outdir):
+    def clean_per_pix(self, fdir, outdir,mode='linear'):
+        # mode = climatology
         Tools().mk_dir(outdir)
         for f in tqdm(Tools().listdir(fdir)):
             dic = Tools().load_npy(fdir + '/' + f)
@@ -2112,7 +2217,12 @@ class Pre_Process:
                 val = dic[pix]
                 val = np.array(val, dtype=np.float)
                 val[val < -9999] = np.nan
-                new_val = Tools().interp_nan(val, kind='linear')
+                if mode == 'linear':
+                    new_val = Tools().interp_nan(val, kind='linear')
+                elif mode == 'climatology':
+                    new_val = Tools().interp_nan_climatology(val)
+                else:
+                    raise UserWarning('mode error')
                 if len(new_val) == 1:
                     continue
                 # plt.plot(val)
